%% multi_NLLS_smk2.m
% This code fits the model to the data (Tx) and uses supporting m-files
%   - multi_OBJ_smk2: Objective function when optimizing local parameters
%   - multi_OBJ_all_smk2: Objective function when optimizing global
%   parameters
%   - multi_smk: definition of ODE (Eq. 1 subject to Eq. 2)
%
%   Global Parameters:
%       - Tx*: nx2 cell array, 
%           - rows are countries identified by string in col 1
%           - col 2 is an array [year, observation]
%       - Lparam*: nx6 array of parameters,
%           - rows are countries
%           - cols are (x0, a, b, u_0, u_inf, delta/tstar)
%       - obj_param: 1x6 vector indicating which vars are being optimized
%       - country: keep track of which country being optimized in Step 1
%       - bounds*: 2x6 array row 1 lb, row 2 ub for 
%           vars (x0, a, b, u_0, u_inf, delta/tstar)
%       - model_no*: 1 - step function, 2 - empirical function
%       - n: number of countries in sample
%       - tot_obs: total number of observations in all countries
%
%   Global Parameters (multi_smk.m):
%       - knowl*: vector for empirical function
%             
%
%   *must be defined before calling multi_NLLS_smk2.m
%
% Inputs:
%   - glob_var: global variables (fixed across countries)
%   - fix_var: fixed variables (vars. to hold constant, i.e. not to fit)
%   - tol: stop when DS_out smaller than tol
%   - max_itn: stop when number of iterations reaches max_itn
%   - plt: option to plot results
%   - plt_int: calculate solution to ODE until year plt_int
%
% Outputs:
%   - S: Value of global objective function
%   - error: [L1 error, L2 error] (error between data and model output)
%   - fit: output of ODE model from initial time until plt_int
%   - xstar: [Equilibria of ODE, stability of ODE]
%       stability == -1 if asymp. stable, +1 if unstable
%   - DS_out: change in global objective function
%
function [S, error, fit, xstar, DS_out] = multi_NLLS_smk2(glob_var, fix_var, tol, max_itn, plt, plt_int)
%% Initialize global variables
global Tx Lparam obj_param country bounds n tot_obs model_no

n = length(Tx);         % number of countries
DS_out = [];            % change in obj. function
tot_obs = 0;            % number of total observations

for i=1:n
    tot_obs = tot_obs + length(Tx{i,2});
end

% Put all observations in a vector together, used to calculate global obj.
% fucntion.
Y = zeros(tot_obs,1);

for i=1:n
    idx = find(Y==0,1);
    Y(idx:idx-1+length(Tx{i,2})) = Tx{i,2}(:,2);
end

% Calculate initial value for global objective function S
obj_param = logical((glob_var==1).*(fix_var==0));

S = multi_OBJ_all_smk2(Lparam(1,obj_param), 1);
S = sum((Y-S).^2);

%% Perform fitting procedure
DS = tol; itn = 0;

while itn <= max_itn && DS >= tol
    % global obj. function from last iteration
    S_temp = S;
    
    %% Step 1: Holding glob_var constant we find optimal 1-glob_var
    %   - Proceed country by country
    obj_param = logical((glob_var==0).*(fix_var==0));
    
    if ~isempty(find(obj_param,1))
        % For each country
        for i=1:n
            country = i;
            
            % Approach for model_no==1 if year is local and non-fixed:
            % scan for optimal t* within bounds
            if obj_param(6)==1 && model_no==1
                % set obj_param(6) = 0 so that lsqcurfit does not change it
                obj_param(6) = 0;
                
                % lower and upper bounds for parameters other than tstar
                lb = bounds(1,obj_param);
                ub = bounds(2,obj_param);
                
                % scan each year and choose the one with lowest error
                for j=bounds(1,6):bounds(2,6)
                    Lparam(i,6) = j;
                
                    [Lparam_temp, tstar_err_temp] = lsqcurvefit(@multi_OBJ_smk2, Lparam(i,obj_param), Tx{i,2}(:,1), Tx{i,2}(:,2), lb, ub);
                    
                    if j==bounds(1,6)
                        tstar = j;
                        tstar_err = tstar_err_temp;
                    elseif tstar_err_temp < tstar_err
                        tstar = j;
                        tstar_err = tstar_err_temp;
                    end
                end
                
                % output tstar with lowest error
                Lparam(i,6) = tstar;
                Lparam(i,obj_param) = lsqcurvefit(@multi_OBJ_smk2, Lparam(i,obj_param), Tx{i,2}(:,1), Tx{i,2}(:,2), lb, ub);
            
                % reset obj_param(6) for next country
                obj_param(6) = 1;
            else
                % set bounds
                lb = bounds(1,obj_param);
                ub = bounds(2,obj_param);
                
                % Perform fit
                Lparam(i,obj_param) = lsqcurvefit(@multi_OBJ_smk2, Lparam(i,obj_param), Tx{i,2}(:,1), Tx{i,2}(:,2), lb, ub);
            end
        end
    end
    
    %% Step 2: Holding 1-glob_var constatnt we find optimal glob_var
    obj_param = logical((glob_var==1).*(fix_var==0));
    
    if ~isempty(find(obj_param,1))
        % Approach for model_no==1 if year is universal and non-fixed:
        % scan years for optimal tstar
        if obj_param(6)==1 && model_no==1
            % set obj_param(6)=0 so lsqcurvefit does not change tstar
            obj_param(6) = 0;

            % lower and upper bounds for parameters other than tstar
            lb = bounds(1,obj_param);
            ub = bounds(2,obj_param);
            
            % scan for optimal tstar
            for j=bounds(1,6):bounds(2,6)
                Lparam(:,6) = j;
                
                % Perform fit
                if ~isempty(find(obj_param,1))
                    [~,tstar_err_temp] = lsqcurvefit(@multi_OBJ_all_smk2, Lparam(1,obj_param), 1, Y, lb, ub);
                else
                    tstar_err_temp = multi_OBJ_all_smk2(Lparam(1,obj_param), 1);
                    tstar_err_temp = sum((Y-tstar_err_temp).^2);
                end
                
                if j==bounds(1,6)
                    tstar = j;
                    tstar_err = tstar_err_temp;
                elseif tstar_err_temp < tstar_err
                    tstar = j;
                    tstar_err = tstar_err_temp;
                end
            end
            
            % update parameters
            Lparam(:,6) = tstar;
            
            if ~isempty(find(obj_param,1))
                Lparam(1,obj_param) = lsqcurvefit(@multi_OBJ_all_smk2, Lparam(1,obj_param), 1, Y, lb, ub);
                Lparam(:,obj_param) = kron(ones(n,1), Lparam(1,obj_param));
            end
        else
            % set bounds
            lb = bounds(1,obj_param);
            ub = bounds(2,obj_param);
            
            % Perform fit
            Lparam(1,obj_param) = lsqcurvefit(@multi_OBJ_all_smk2, Lparam(1,obj_param), 1, Y, lb, ub);
            Lparam(:,obj_param) = kron(ones(n,1), Lparam(1,obj_param));
        end
    end
    
    % calculate global objective function
    S = multi_OBJ_all_smk2(Lparam(1,obj_param), 1);
    S = sum((Y-S).^2);
    
    % calculate change in global objective function
    DS = abs(S-S_temp);
    DS_out = [DS_out; DS];
    
    % update iteration number
    itn = itn+1;
end

%% Output
obj_param = logical((glob_var==0).*(fix_var==0));

fit = cell(n,1);
error = zeros(n,2);
xstar = zeros(n,2);

if plt==1 
    hold on
    names=cell(2*n,1);
end

for i=1:n
    country = i;
    
    % record output from ODE model with fitted parameters
    fit{i} = multi_OBJ_smk2(Lparam(i,obj_param),...
        [Tx{i,2}(:,1); (Tx{i,2}(end,1)+1:plt_int)']);

    fit{i} = [[Tx{i,2}(:,1); (Tx{i,2}(end,1)+1:plt_int)'], fit{i}];
    
    if plt == 1
        plot(fit{i}(:,1),fit{i}(:,2))
        scatter(Tx{i,2}(:,1),Tx{i,2}(:,2))
        names{2*i-1} = Tx{i,1};
        names{2*i} = Tx{i,1};
    end
    
    % record L1 and L2 error
    err = abs(Tx{i,2}(:,2) - fit{i}(1:length(Tx{i,2}),2));
    error(i,1) = sum(err);
    error(i,2) = sum(err.^2);
    
    % Stability of equilibria
    if Lparam(i,2) < 1
        xstar(i,2) = -1;
    elseif Lparam(i,2) > 1
        xstar(i,2) = 1;
    end
    
    % Calculate equilibria
    xstar(i,1) = (1 + ((1-Lparam(i,5))/Lparam(i,5))^(Lparam(i,2)/(1-Lparam(i,2))) )^(-1);
end

if plt == 1
    legend(names)
    xlabel('Year','FontSize',14)
    ylabel('Smoking prevalence (x)','FontSize',14)
    legend('hide')
end